import hell2;
import token;
import player;
import particle;
import enemy;


/**
 * メインシーン
 */
class SceneMain : IScene
{
public:
	enum
	{
		STATE_INIT,
		STATE_START_WAIT,
		STATE_MAIN,
		STATE_GAMECLEAR,
		STATE_GAMECLEAR_WAIT,
	}
	const int STAGE_MAX       = 8;
	const int TIMER_GAMECLEAR = 30;
	const int TIMER_START_WAIT = 180;
public:
	int state;
	int timer;
	Player player;
	int stage;
	int tickStart;
	int tickNow;
	int tLimit;
	bool bDanger;
public:
	this()
	{
		player          = new Player();
		Bullet.parent   = new TokenManager!(Bullet)(256);
		Shot.parent     = new TokenManager!(Shot)(8);
		Enemy.parent    = new TokenManager!(Enemy)(64);
		Particle.parent = new TokenManager!(Particle)(256);
		Item.parent     = new TokenManager!(Item)(128);
	}
	override void init()
	{
		reset();
	}
	
	void reset()
	{
		state = STATE_INIT;
		Bullet.parent.vanish();
		Shot.parent.vanish();
		Enemy.parent.vanish();
		Particle.parent.vanish();
		Item.parent.vanish();
		player.create(screenW/2, screenH*8/10, 16, 16);
		timer = 0;
		stage = 1;
		tLimit = 0;
		player.start();
		bDanger = false;
	}
	override void update()
	{
		player.update();
		Shot.parent.update();
		Particle.parent.update();

		switch(state)
		{
		case STATE_INIT:
			// 初期化
			Hell_randSeed(0, 1);
			tickStart = 0;
			tickNow   = 0;
			switch(stage)
			{
			case 1:
				tLimit = 59*1000;
				for(int i = 0; i < 3; i++)
				{
					Enemy.addEx(1);
				}
				break;
			case 2:
				tLimit = 30*1000;
				for(int i = 0; i < 10; i++)
				{
					Enemy.addEx(1);
				}
				break;
			case 3:
				tLimit = 5*1000;
				for(int i = 0; i < 2; i++)
				{
					Enemy.addEx(2);
				}
				break;
			case 4:
				tLimit = 10*1000;
				for(int i = 0; i < 2; i++)
				{
					Enemy.addEx(1);
				}
				for(int i = 0; i < 8; i++)
				{
					Enemy.addEx(2);
				}
				break;
				
			case 5:
				tLimit = 7*1000;
				for(int i = 0; i < 5; i++)
				{
					Enemy.addEx(1);
				}
				for(int i = 0; i < 2; i++)
				{
					Enemy.addEx(3);
				}
				break;
				
			case 6:
				tLimit = 10*1000;
				for(int i = 0; i < 8; i++)
				{
					Enemy.addEx(3);
				}
				break;
				
			case 7:
				tLimit = 8*1000;
				for(int i = 0; i < 8; i++)
				{
					Enemy.addEx(4);
				}
				break;
				
			case 8:
				tLimit = 7*1000;
				for(int i = 0; i < 6; i++)
				{
					Enemy.addEx(5);
				}
				break;
				
			}
			state = STATE_START_WAIT;
			timer = TIMER_START_WAIT;
			break;
			
		case STATE_START_WAIT:
			timer--;
			if(timer < 1 || Hell_isPushEnter())
			{
				bDanger   = false;
				state = STATE_MAIN;
				tickStart = Hell_getTicks();
				tickNow   = tickStart;
				player.start();
				timer = 0;
			}
			break;
		case STATE_MAIN:
			// メイン
			timer++;
			updateMain();
			break;
		case STATE_GAMECLEAR:
			// ゲームクリア
			if(timer > 0)
			{
				float t = (240 - timer)/240.0;
				Hell_drawRectEx(320, 240, 320, 240*t, 0, 0, 0xff, 0xff, 0xff, 0xff);
				timer = timer*95/100;
			}
			if(timer < 1)
			{
				state = STATE_GAMECLEAR_WAIT;
			}
			break;
		case STATE_GAMECLEAR_WAIT:
			// クリアウェイト
			Shot.parent.update();
			Particle.parent.update();
//			if(Hell_isPushEnter())
			{
				SceneManager.change("end");
				SceneManager.get().init();
			}
			break;
		}
		
		// 描画
		draw();
	}
	void updateMain()
	{
		// 色々更新
		{
			tickNow = Hell_getTicks();
			int tickPast = tickNow - tickStart;
			int limit = tLimit - tickPast;
			if(limit < 3*1000 && bDanger == false)
			{
				// 時間がないよ
				bDanger = true;
				Hell_playWAV("danger");
			}
		}
		// キャラオブジェクト
		Enemy.parent.update();
		Shot.parent.update();
		Bullet.parent.update();
		Item.parent.update();
		Particle.parent.update();
		
		// 当たり判定
		bool destroy = false;
		

// デバッグ用
if(Hell_isPushCancel())
{
	Enemy.destroyAll();
}
		
		// vs 自機の弾
		foreach(shot; Shot.parent.getArray())
		{
			if(!shot.exist)         { continue; }
			// 敵
			foreach(enemy; Enemy.parent.getArray())
			{
				if(!enemy.exist) { continue; }
				if(!shot.isHit2(enemy)) { continue; }
				shot.bLock = true;
				if(!shot.isHitStart())  { continue; }
				game.addScore(enemy.getScore());
				shot.vanish();
				enemy.vanish();
				// 敵破壊
				destroy = true;
			}
		}
		if(destroy) { Hell_playWAV("bomb", 0, 2); }

		// 敵の弾
		foreach(bullet; Bullet.parent.getArray())
		{
			if(!bullet.exist) continue;
			
		}
		
		// 自機
		if(player.exist)
		{
			// アイテム
			foreach(item; Item.parent.getArray())
			{
				if(!item.exist) continue;
				if(item.isHit2(player))
				{
					player.getItem(item);
					item.exist = false;
				}
			}
			
			// vs 敵の弾
			foreach(bullet; Bullet.parent.getArray())
			{
				if(!bullet.exist) continue;
				if(bullet.isHit2(player))
				{
					break;
				}
			}
		}
		
		int tickPast = tickNow - tickStart;
		if(Enemy.parent.count() == 0 || tickPast >= tLimit)
		{
			Enemy.destroyAll();
			if(tickPast >= tLimit)
			{
				Hell_playWAV("timeup");
			}
			if(stage >= STAGE_MAX)
			{
				// ゲームクリア
				state = STATE_GAMECLEAR;
				timer = TIMER_GAMECLEAR;
				Hell_stopBgm(3000);
			}
			else
			{
				// 次のステージへ
				stage++;
				state = STATE_INIT;
			}
		}
		
	}
	void draw()
	{
		drawBack();
		
		Item.parent.draw();
		Enemy.parent.draw();
		if(player.exist)
		{
			player.draw();
		}
		Shot.parent.draw();
		Bullet.parent.draw();
		Particle.parent.draw();

		if(state == STATE_START_WAIT)
		{
			Hell_drawFontEx(100, 200, 1.0f, 0xff, 0xff, 0xff, 0xff, "   STAGE %d", stage);
			Hell_drawFontEx(100, 280, .7f, Hell_rand(0xff), Hell_rand(0xff), 0xff, 0xff,  "   LIMIT: %02d sec", tLimit/1000%60);
		}
		drawFrame();
		drawTarget();
		drawScore();
		drawDebug();
	}
	
	// 背景描画
	void drawBack()
	{
		Hell_drawRect(0, 0, 640, 480, 0, 0, 0x20, 0x20, 0x20);
	}
	
	// 枠表示
	void drawFrame()
	{
		int c = 0x20;
		Hell_setAlpha(HELL_ALPHA_NORMAL);
		Hell_drawRect(0, 0, 640, 64, 0, 0, c, c, c);
		Hell_setAlpha(HELL_ALPHA_ADD);
		c = cast(int)(0x40*Hell_sinEx(timer*2))+0x7f;
		Hell_drawRect(0, 0, 640, 64, 0, 0, c, c, c);
		Hell_setAlpha(HELL_ALPHA_NORMAL);
	}
	
	// 目標数表示
	void drawTarget()
	{
		int x   = 16;
		int y   = 24;
		foreach(e; Enemy.parent.getArray())
		{
			if(!e.exist) { continue; }
			e.Draw(e, x, y, 0.5f);
			x += 32;
		}
	}
	
	// スコア表示
	void drawScore()
	{
		int x = 360;
		int y = 5;
		Hell_drawFontEx(x, y,    .5f, 0xff, 0xff, 0xff, 0xff, "SCORE   :%8d", game.score);
		Hell_drawFontEx(x, y+30, .5f, 0xff, 0xff, 0xff, 0xff, "HI-SCORE:%8d", game.hiscore);
		int tickPast = tickNow - tickStart;
		int limit = tLimit - tickPast;
		if(limit < 0) { limit = 0; }
		if(limit < 3*1000)
		{
			Hell_drawFontEx(x, y+60, .5f, 0xff, 0x00, 0x00, 0xff, "LIMIT   :  %02d:%03d", limit/1000%60, limit%1000);
		}
		else
		{
			Hell_drawFontEx(x, y+60, .5f, 0xff, 0xff, 0xff, 0xff, "LIMIT   :  %02d:%03d", limit/1000%60, limit%1000);
		}
	}
	
	// デバッグ表示
	void drawDebug()
	{
		Hell_drawFontEx(0, 400,  .4f, 0xff, 0xff, 0xff, 0xff, "Shot:%d", Shot.parent.count());
		Hell_drawFontEx(0, 424, .4f, 0xff, 0xff, 0xff, 0xff, "Enem:%d", Enemy.parent.count());
	}
}
